import { generateFontFace } from '@fontsource-utils/generate';

const cleanPaths = (str) => str.replace("[", "").replace("]", "");
const makeFontDownloadPath = (fontDir, fontId, subset, weight, style, extension) => cleanPaths(
  `${fontDir}/files/${fontId}-${subset}-${weight}-${style}.${extension}`
);
const makeVariableFontDownloadPath = (fontDir, fontId, subset, axes, style) => cleanPaths(
  `${fontDir}/files/${fontId}-${subset}-${axes.toLowerCase()}-${style}.woff2`
);
const makeFontFilePath = (fontId, subset, weight, style, extension) => cleanPaths(`./files/${fontId}-${subset}-${weight}-${style}.${extension}`);
const makeVariableFontFilePath = (fontId, subset, axes, style) => cleanPaths(`./files/${fontId}-${subset}-${axes}-${style}.woff2`);
const findClosest = (arr, num) => {
  const indexArr = arr.map((weight) => Math.abs(Number(weight) - num));
  const min = Math.min(...indexArr);
  const closest = arr[indexArr.indexOf(min)];
  return closest;
};
const licenseMap = {
  "apache license, version 2.0": "Apache-2.0",
  "sil open font license, 1.1": "OFL-1.1",
  "ubuntu font license, 1.0": "UFL-1.0",
  "mit license": "MIT",
  "cc0-1.0": "CC0-1.0"
};
const licenseShort = (license) => licenseMap[license.toLowerCase()];
const sassVar = (key, value) => `$${key}: ${value} !default;
`;
const assertNever = (value) => {
  throw new Error(`Unhandled union member: ${JSON.stringify(value)}`);
};

const generateV1CSS = (metadata, makeFontFilePath, tag) => {
  const cssGenerate = [];
  const { id, family, styles, weights, subsets, variants } = metadata;
  for (const subset of subsets) {
    const cssSubset = [];
    const cssSubsetItalic = [];
    for (const weight of weights) {
      for (const style of styles) {
        if (style in variants[weight]) {
          const fontObj = {
            family,
            style,
            display: "swap",
            weight,
            src: [
              {
                url: makeFontFilePath(
                  tag ?? id,
                  subset,
                  String(weight),
                  style,
                  "woff2"
                ),
                format: "woff2"
              },
              {
                url: makeFontFilePath(
                  tag ?? id,
                  subset,
                  String(weight),
                  style,
                  "woff"
                ),
                format: "woff"
              }
            ],
            comment: `${tag ?? id}-${subset}-${weight}-${style}`
          };
          const css = generateFontFace(fontObj);
          if (style === "normal") {
            cssGenerate.push({
              filename: `${subset}-${weight}.css`,
              css
            });
            cssSubset.push(css);
          } else {
            cssGenerate.push({
              filename: `${subset}-${weight}-${style}.css`,
              css
            });
            cssSubsetItalic.push(css);
          }
        }
      }
    }
    cssGenerate.push({
      filename: `${subset}.css`,
      css: cssSubset.join("\n\n")
    });
    if (cssSubsetItalic.length > 0) {
      cssGenerate.push({
        filename: `${subset}-italic.css`,
        css: cssSubsetItalic.join("\n\n")
      });
    }
  }
  return cssGenerate;
};
const generateV2CSS = (metadata, makeFontFilePath, tag) => {
  const cssGenerate = [];
  const { id, family, styles, weights, variants, unicodeRange } = metadata;
  const indexWeight = findClosest(weights, 400);
  const unicodeKeys = Object.keys(unicodeRange);
  for (const weight of weights) {
    for (const style of styles) {
      const cssStyle = [];
      for (const subset of unicodeKeys) {
        if (style in variants[weight]) {
          const fontObj = {
            family,
            style,
            display: "swap",
            weight,
            unicodeRange: unicodeRange[subset],
            src: [
              {
                url: makeFontFilePath(
                  tag ?? id,
                  subset,
                  String(weight),
                  style,
                  "woff2"
                ),
                format: "woff2"
              },
              {
                url: makeFontFilePath(
                  tag ?? id,
                  subset,
                  String(weight),
                  style,
                  "woff"
                ),
                format: "woff"
              }
            ],
            comment: `${tag ?? id}-${subset}-${weight}-${style}`
          };
          const css = generateFontFace(fontObj);
          cssStyle.push(css);
        }
      }
      if (style in variants[weight]) {
        if (style === "normal") {
          cssGenerate.push({
            filename: `${weight}.css`,
            css: cssStyle.join("\n\n")
          });
          if (weight === indexWeight) {
            cssGenerate.push({
              filename: "index.css",
              css: cssStyle.join("\n\n")
            });
          }
        } else {
          cssGenerate.push({
            filename: `${weight}-${style}.css`,
            css: cssStyle.join("\n\n")
          });
        }
      }
    }
  }
  return cssGenerate;
};
const generateVariableCSS = (metadata, variableMeta, makeFontFilePath, tag) => {
  const { id, family, unicodeRange, weights } = metadata;
  const { axes, variants } = variableMeta;
  const cssGenerate = [];
  let indexCSS = "";
  for (const axesKey of Object.keys(variants)) {
    const variant = variants[axesKey];
    const styles = Object.keys(variant);
    const axesLower = axesKey.toLowerCase();
    const variableOpts = {
      wght: axes.wght
    };
    if (axesKey === "standard" || axesKey === "full" || axesKey === "wdth")
      variableOpts.stretch = axes.wdth;
    if (axesKey === "standard" || axesKey === "full" || axesKey === "slnt")
      variableOpts.slnt = axes.slnt;
    for (const style of styles) {
      const cssStyle = [];
      for (const subset of Object.keys(variant[style])) {
        const fontObj = {
          family,
          style,
          display: "swap",
          weight: findClosest(weights, 400),
          unicodeRange: unicodeRange[subset],
          variable: variableOpts,
          src: [
            {
              url: makeFontFilePath(tag ?? id, subset, axesLower, style),
              format: "woff2-variations"
            }
          ],
          comment: `${tag ?? id}-${subset}-${axesLower}-${style}`
        };
        const css2 = generateFontFace(fontObj);
        cssStyle.push(css2);
      }
      const filename = style === "normal" ? `${axesLower}.css` : `${axesLower}-${style}.css`;
      const css = cssStyle.join("\n\n");
      cssGenerate.push({
        filename,
        css
      });
      if (axesKey === "wght" && (style === "normal" || styles.length === 1))
        indexCSS = css;
      if (indexCSS === "" && axesKey === "opsz")
        indexCSS = css;
    }
  }
  cssGenerate.push({
    filename: "index.css",
    css: indexCSS
  });
  return cssGenerate;
};
const generateIconStaticCSS = (metadata, makeFontFilePath, tag) => {
  const cssGenerate = [];
  const { id, family, styles, weights, subsets, variants } = metadata;
  const indexWeight = findClosest(weights, 400);
  for (const subset of subsets) {
    const cssSubset = [];
    for (const weight of weights) {
      for (const style of styles) {
        if (style in variants[weight]) {
          const fontObj = {
            family,
            style,
            display: "swap",
            weight,
            src: [
              {
                url: makeFontFilePath(
                  tag ?? id,
                  subset,
                  String(weight),
                  style,
                  "woff2"
                ),
                format: "woff2"
              },
              {
                url: makeFontFilePath(
                  tag ?? id,
                  subset,
                  String(weight),
                  style,
                  "woff"
                ),
                format: "woff"
              }
            ],
            comment: `${tag ?? id}-${subset}-${weight}-${style}`
          };
          const css = generateFontFace(fontObj);
          if (style === "normal") {
            cssGenerate.push(
              {
                filename: `${weight}.css`,
                css
              },
              {
                filename: `${subset}-${weight}.css`,
                css
              }
            );
          } else {
            cssGenerate.push(
              {
                filename: `${weight}-italic.css`,
                css
              },
              {
                filename: `${subset}-${weight}-italic.css`,
                css
              }
            );
          }
          cssSubset.push(css);
        }
      }
      if (weight === indexWeight) {
        cssGenerate.push({
          filename: "index.css",
          css: cssSubset.join("\n\n")
        });
      }
      cssGenerate.push({
        filename: `${subset}.css`,
        css: cssSubset.join("\n\n")
      });
    }
  }
  return cssGenerate;
};
const generateIconVariableCSS = (metadata, makeFontFilePath, tag) => {
  const cssGenerate = [];
  const { id, family, variants, axes } = metadata;
  let indexCSS = "";
  for (const axesKey of Object.keys(variants)) {
    const variant = variants[axesKey];
    const styles = Object.keys(variant);
    const axesLower = axesKey.toLowerCase();
    const variableOpts = {
      wght: axes.wght
    };
    if (axesKey === "standard" || axesKey === "full" || axesKey === "wdth")
      variableOpts.stretch = axes.wdth;
    if (axesKey === "standard" || axesKey === "full" || axesKey === "slnt")
      variableOpts.slnt = axes.slnt;
    for (const style of styles) {
      const cssStyle = [];
      for (const subset of Object.keys(variant[style])) {
        const fontObj = {
          family,
          style,
          display: "swap",
          weight: Number(axes.wght.default),
          variable: variableOpts,
          src: [
            {
              url: makeFontFilePath(tag ?? id, subset, axesLower, style),
              format: "woff2-variations"
            }
          ],
          comment: `${tag ?? id}-${subset}-${axesLower}-${style}`
        };
        const css2 = generateFontFace(fontObj);
        cssStyle.push(css2);
      }
      const filename = style === "normal" ? `${axesLower}.css` : `${axesLower}-${style}.css`;
      const css = cssStyle.join("\n\n");
      cssGenerate.push({
        filename,
        css
      });
      if (axesKey === "wght")
        indexCSS = css;
      if (!indexCSS && axesKey === "opsz")
        indexCSS = css;
    }
  }
  cssGenerate.push({
    filename: "index.css",
    css: indexCSS
  });
  return cssGenerate;
};

export { generateIconVariableCSS as a, generateV1CSS as b, generateV2CSS as c, generateVariableCSS as d, makeVariableFontFilePath as e, findClosest as f, generateIconStaticCSS as g, makeFontDownloadPath as h, makeVariableFontDownloadPath as i, assertNever as j, licenseShort as l, makeFontFilePath as m, sassVar as s };
